"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortKeysRule = void 0;
const utils_1 = require("@typescript-eslint/utils");
exports.sortKeysRule = {
    meta: {
        type: "suggestion",
        docs: {
            description: "Enforce sorted keys in objects",
        },
        fixable: "code",
        messages: {
            sortKeys: 'Expected object keys to be in {{ order }} order. "{{ thisName }}" should be before "{{ prevName }}".',
        },
        schema: [
            {
                type: "string",
                enum: ["asc", "desc"],
            },
            {
                type: "object",
                properties: {
                    caseSensitive: { type: "boolean" },
                    natural: { type: "boolean" },
                    minKeys: { type: "number" },
                    allowLineSeparatedGroups: { type: "boolean" },
                },
                additionalProperties: false,
            },
        ],
    },
    defaultOptions: [
        "asc",
        {
            caseSensitive: true,
            natural: false,
            minKeys: 0,
            allowLineSeparatedGroups: false,
        },
    ],
    create(context) {
        const sourceCode = context.getSourceCode();
        const [order, configOptions = {}] = context.options;
        const caseSensitive = configOptions.caseSensitive ?? true;
        const natural = configOptions.natural ?? false;
        const minKeys = configOptions.minKeys ?? 0;
        const allowLineSeparatedGroups = configOptions.allowLineSeparatedGroups ?? false;
        const isAscending = order === "asc";
        const compareKeys = (a, b) => {
            if (!caseSensitive) {
                a = a.toLowerCase();
                b = b.toLowerCase();
            }
            return natural
                ? (isAscending ? 1 : -1) *
                    a.localeCompare(b, undefined, { numeric: true })
                : (isAscending ? a < b : b < a)
                    ? -1
                    : 1;
        };
        return {
            ObjectExpression(node) {
                const properties = node.properties.filter((prop) => prop.type === utils_1.AST_NODE_TYPES.Property);
                if (properties.length < minKeys)
                    return;
                properties.forEach((prop, index) => {
                    if (index === 0)
                        return;
                    const prevProp = properties[index - 1];
                    const thisName = getKey(prop);
                    const prevName = getKey(prevProp);
                    if (!thisName || !prevName)
                        return;
                    if (compareKeys(prevName, thisName) > 0) {
                        context.report({
                            node: prop.key,
                            messageId: "sortKeys",
                            data: { order, thisName, prevName },
                            fix: (fixer) => {
                                const sortedProps = [...properties].sort((a, b) => {
                                    const aName = getKey(a) || "";
                                    const bName = getKey(b) || "";
                                    return compareKeys(aName, bName);
                                });
                                const fixedCode = sortedProps
                                    .map((p) => sourceCode.getText(p))
                                    .join(", ");
                                return fixer.replaceTextRange([node.range[0], node.range[1]], `{ ${fixedCode} }`);
                            },
                        });
                    }
                });
                function getKey(prop) {
                    if (prop.key.type === utils_1.AST_NODE_TYPES.Identifier)
                        return prop.key.name;
                    if (prop.key.type === utils_1.AST_NODE_TYPES.Literal &&
                        typeof prop.key.value === "string") {
                        return prop.key.value;
                    }
                    return null;
                }
            },
        };
    },
};
