"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortKeysRule = void 0;
exports.sortKeysRule = {
    meta: {
        type: "suggestion",
        docs: {
            description: "Enforce sorted keys in objects",
        },
        fixable: "code",
        messages: {
            sortKeys: 'Expected object keys to be in {{ order }} order. "{{ thisName }}" should be before "{{ prevName }}".',
        },
        schema: [
            {
                type: "string",
                enum: ["asc", "desc"],
            },
            {
                type: "object",
                properties: {
                    caseSensitive: { type: "boolean" },
                    natural: { type: "boolean" },
                    minKeys: { type: "number" },
                    allowLineSeparatedGroups: { type: "boolean" },
                },
                additionalProperties: false,
            },
        ],
    },
    defaultOptions: [
        "asc",
        {
            caseSensitive: true,
            natural: false,
            minKeys: 0,
            allowLineSeparatedGroups: false,
        },
    ],
    create(context) {
        const [order, configOptions = {}] = context.options;
        const caseSensitive = configOptions.caseSensitive ?? true;
        const natural = configOptions.natural ?? false;
        const minKeys = configOptions.minKeys ?? 0;
        const allowLineSeparatedGroups = configOptions.allowLineSeparatedGroups ?? false;
        const isAscending = order === "asc";
        function compareKeys(a, b) {
            if (!caseSensitive) {
                a = a.toLowerCase();
                b = b.toLowerCase();
            }
            if (natural) {
                return isAscending
                    ? a.localeCompare(b, undefined, { numeric: true })
                    : b.localeCompare(a, undefined, { numeric: true });
            }
            return isAscending ? (a < b ? -1 : 1) : a > b ? -1 : 1;
        }
        function hasBlankLineBetween(prevProp, prop) {
            return prevProp.loc.end.line < prop.loc.start.line - 1;
        }
        function groupPropertiesByBlankLine(properties) {
            const groups = [];
            let currentGroup = [];
            properties.forEach((prop, index) => {
                if (index > 0 &&
                    allowLineSeparatedGroups &&
                    hasBlankLineBetween(properties[index - 1], prop)) {
                    groups.push(currentGroup);
                    currentGroup = [];
                }
                currentGroup.push(prop);
            });
            if (currentGroup.length) {
                groups.push(currentGroup);
            }
            return groups;
        }
        return {
            ObjectExpression(node) {
                const properties = node.properties.filter((prop) => prop.type === "Property");
                if (properties.length < minKeys)
                    return;
                const groupedProperties = groupPropertiesByBlankLine(properties);
                groupedProperties.forEach((group) => {
                    group.forEach((prop, index) => {
                        if (index === 0)
                            return;
                        const prevProp = group[index - 1];
                        const thisName = prop.key.type === "Identifier"
                            ? prop.key.name
                            : prop.key.type === "Literal" &&
                                typeof prop.key.value === "string"
                                ? prop.key.value
                                : "";
                        const prevName = prevProp.key.type === "Identifier"
                            ? prevProp.key.name
                            : prevProp.key.type === "Literal" &&
                                typeof prevProp.key.value === "string"
                                ? prevProp.key.value
                                : "";
                        if (compareKeys(prevName, thisName) > 0) {
                            context.report({
                                node: prop.key,
                                messageId: "sortKeys",
                                data: { order, thisName, prevName },
                                fix: (fixer) => {
                                    const sortedProps = [...group].sort((a, b) => {
                                        const aName = a.key.type === "Identifier"
                                            ? a.key.name
                                            : a.key.type === "Literal" &&
                                                typeof a.key.value === "string"
                                                ? a.key.value
                                                : "";
                                        const bName = b.key.type === "Identifier"
                                            ? b.key.name
                                            : b.key.type === "Literal" &&
                                                typeof b.key.value === "string"
                                                ? b.key.value
                                                : "";
                                        return compareKeys(aName, bName);
                                    });
                                    const sourceCode = context.getSourceCode();
                                    const fixedCode = sortedProps
                                        .map((p) => sourceCode.getText(p))
                                        .join(", ");
                                    return fixer.replaceTextRange([group[0].range[0], group[group.length - 1].range[1]], `{ ${fixedCode} }`);
                                },
                            });
                        }
                    });
                });
            },
        };
    },
};
